package com.engine.kr.entity.kq;

import org.apache.commons.lang3.StringUtils;
import weaver.conn.RecordSet;

import java.util.HashMap;
import java.util.Map;

/**
 * 加班流程转换类
 *
 * @author:dxfeng
 * @createTime: 2023/05/23
 * @version: 1.0
 */
public class KqActionBO {

    /**
     * 根据人员ID获取人员名称
     *
     * @param userId
     * @return
     */
    public static String getUserName(String userId) {
        String userName = "";
        if (StringUtils.isBlank(userId)) {
            return userName;
        }
        RecordSet rs = new RecordSet();
        rs.executeQuery("select lastname from hrmresource where id = ? ", userId);
        if (rs.next()) {
            userName = rs.getString("lastname");
        }
        return userName;
    }

    /**
     * 根据人员ID获取人员工号
     *
     * @param userId
     * @return
     */
    public static String getWorkCode(String userId) {
        String workCode = "";
        if (StringUtils.isBlank(userId)) {
            return workCode;
        }
        RecordSet rs = new RecordSet();
        rs.executeQuery("select workcode from hrmresource where id = ? ", userId);
        if (rs.next()) {
            workCode = rs.getString("workcode");
        }
        return workCode;
    }

    /**
     * 获取请假类型代码
     *
     * @param leaveType 流程请假类型
     * @return 请假类型代码
     */
    public static String getLeaveTypeCode(String leaveType) {
        String leaveTypeCode = "";
        if (StringUtils.isBlank(leaveType)) {
            return leaveTypeCode;

        }
        RecordSet rs = new RecordSet();
        rs.executeQuery("select kqxtzdz from uf_jqzdpp where lczd =?", leaveType);
        if (rs.next()) {
            leaveTypeCode = rs.getString("kqxtzdz");
        }
        return leaveTypeCode;
    }

    /**
     * 获取加班类型
     *
     * @param workType 流程加班类型
     * @return 加班类型代码
     */
    public static String getWorkTypeCode(String workType) {
        String workTypeCode = "";
        if (StringUtils.isBlank(workType)) {
            return workTypeCode;
        }
        switch (workType) {
            case "0":
                workTypeCode = "JB01";
                break;
            case "1":
                workTypeCode = "JB02";
                break;
            case "2":
                workTypeCode = "JB03";
                break;
            case "3":
                workTypeCode = "JB20";
                break;
            case "4":
                workTypeCode = "JB30";
                break;
            default:
                break;
        }
        return workTypeCode;
    }

    /**
     * 获取流程创建、最后操作时间
     *
     * @param requestId
     * @return
     */
    public static Map<String, String> getOperateDateTime(String requestId) {
        Map<String, String> map = new HashMap<>(2);
        RecordSet rs = new RecordSet();
        rs.executeQuery("select createdate,createtime,lastoperatedate,lastoperatetime from workflow_requestbase where requestid = ? ", requestId);
        if (rs.next()) {
            String createdate = rs.getString("createdate");
            String createtime = rs.getString("createtime");
            String lastoperatedate = rs.getString("lastoperatedate");
            String lastoperatetime = rs.getString("lastoperatetime");
            map.put("addDay", createdate + " " + createtime);
            map.put("cfmDay", lastoperatedate + " " + lastoperatetime);
        }
        return map;
    }
}
