package weaver.interfaces.kr.workflow.action;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.engine.kr.entity.kq.KqActionBO;
import com.engine.kr.entity.kq.KqGlobal;
import com.engine.kr.enums.KqInteractiveEnum;
import com.engine.kr.service.impl.KqInteractiveServiceImpl;
import weaver.general.BaseBean;
import weaver.interfaces.workflow.action.Action;
import weaver.soa.workflow.request.MainTableInfo;
import weaver.soa.workflow.request.Property;
import weaver.soa.workflow.request.RequestInfo;

import java.util.HashMap;
import java.util.Map;

/**
 * 康瑞-出差申请流程-Action
 *
 * @author wangj
 * @version 1.00版本
 * @Date 2023/6/5
 */
public class KrCCLeaveCreateAction implements Action {
    @Override
    public String execute(RequestInfo requestInfo) {
        MainTableInfo mainTableInfo = requestInfo.getMainTableInfo();
        String personId = "";
        String startDate = "";
        String startTime = "";
        String endDate = "";
        String endTime = "";
        String leaveDuration = "";
        String leaveReason = "";
        String billNo = "";

        Property[] properties = mainTableInfo.getProperty();
        for (Property property : properties) {
            String name = property.getName();
            String value = property.getValue();
            switch (name) {
                case "resourceId":
                    personId = value;
                    break;
                case "lcbh":
                    billNo = value;
                    break;
                case "ksrq":
                    startDate = value;
                    break;
                case "kssj":
                    startTime = value;
                    break;
                case "jsrq":
                    endDate = value;
                    break;
                case "jssj":
                    endTime = value;
                    break;
                case "ccsc":
                    leaveDuration = value;
                    break;
                case "ccsy":
                    leaveReason = value;
                    break;

                default:
                    break;
            }

        }
        Map<String, String> operateDateTime = KqActionBO.getOperateDateTime(requestInfo.getRequestid());
        Map<String, Object> params = new HashMap<>();
        params.put("token", new KqInteractiveServiceImpl().getToken());
        params.put("personId", KqActionBO.getWorkCode(personId));
        params.put("billNo", billNo);
        params.put("dayoffType", KqActionBO.getLeaveTypeCode("13"));
        params.put("beginDate", startDate);
        params.put("beginTime", startTime);
        params.put("endDate", endDate);
        params.put("endTime", endTime);
        params.put("billHours", leaveDuration);
        params.put("runsNo", null);
        params.put("inout", null);
        params.put("incHoli", null);
        params.put("dayoffReason", leaveReason);
        params.put("addBy", requestInfo.getCreatorid());
        params.put("addDay", operateDateTime.get("addDay"));
        params.put("cfmBy", KqActionBO.getUserName(requestInfo.getLastoperator()));
        params.put("cfmDay", operateDateTime.get("addDay"));
        // 1新增、2变动、3取消
        params.put("billStat", 1);

        new BaseBean().writeLog("########出差流程,requestid={" + requestInfo.getRequestid() + "]，传参:"+ JSON.toJSONString(params));

        String returnStr = HttpUtil.post(KqGlobal.POST_URL + KqInteractiveEnum.LEAVE_CREATE.getPostPath(), params);
        new BaseBean().writeLog("########出差流程,requestid={" + requestInfo.getRequestid() + "]，接口响应数据:"+ returnStr);
        JSONObject jsonObject = JSONObject.parseObject(returnStr);
        String code = jsonObject.getString("code");
        if (KqGlobal.SUC_CODE.equals(code)) {
            return SUCCESS;
        }
        requestInfo.getRequestManager().setMessagecontent(jsonObject.getString("msg"));
        return FAILURE_AND_CONTINUE;
    }


}
